local SCALE = 50
local NUMBALLS = 600
local BALLRADIUS = 8
local UNMARKED, MARKED = { .5, .5, .6, 1 }, { 0, .7, 1, 1 }
local OBSX, OBSY, OBSR = 400, 200, 50

local MARK = {}
for _, i in ipairs({
	7, 8, 18, 34, 56, 60, 77, 86, 89, 91, 95, 96, 99, 106, 113, 114, 116, 117, 119, 121, 123, 124, 128, 137, 142, 143,
	145, 159, 161, 164, 175, 182, 187, 191, 193, 204, 205, 208, 209, 222, 225, 226, 228, 237, 239, 244, 245, 251, 253,
	266, 267, 275, 279, 280, 282, 289, 295, 312, 323, 330, 331, 332, 336, 355, 358, 364, 377, 385, 390, 392, 395, 402,
	412, 420, 421, 428, 433, 434, 437, 442, 447, 454, 456, 481, 487, 490, 495, 501, 502, 504, 506, 510, 517, 518, 519,
	525, 529, 533, 542, 553, 565, 579
}) do
	MARK[i] = MARKED
end

local phy, gfx = love.physics, love.graphics
local rng = love.math.newRandomGenerator(12345)

local world = phy.newWorld(0, SCALE * 9.81, true)

phy.newFixture(phy.newBody(world,    0,  600, 'static'), phy.newEdgeShape(0, 0, 800, 0))
phy.newFixture(phy.newBody(world,    0,    0, 'static'), phy.newEdgeShape(0, 0, 0, 600))
phy.newFixture(phy.newBody(world,  800,    0, 'static'), phy.newEdgeShape(0, 0, 0, 600))
phy.newFixture(phy.newBody(world, OBSX, OBSY, 'static'), phy.newCircleShape(OBSR))

local ballShape = phy.newCircleShape(BALLRADIUS)
local balls = {}

local function nextBall()
	local x, y = (400 - BALLRADIUS) + rng:random(BALLRADIUS * 2), -rng:random(BALLRADIUS * 4)
	local body = phy.newBody(world, x, y, 'dynamic')
	local fix = phy.newFixture(body, ballShape)
	fix:setRestitution(.7)
	table.insert(balls, { body, MARK[#balls + 1] or UNMARKED })
end

local accum = 0
function love.update(dt)
	accum = accum + math.min(dt, 1 / 20)
	while accum >= 1 / 60 do
		world:update(1 / 60)
		accum = accum - 1 / 60
		if #balls < NUMBALLS then
			nextBall()
		end
	end
end

local sprite = gfx.newCanvas(BALLRADIUS * 2, BALLRADIUS * 2)
sprite:renderTo(function()
	gfx.circle('fill', BALLRADIUS, BALLRADIUS, BALLRADIUS)
	gfx.circle('line', BALLRADIUS, BALLRADIUS, BALLRADIUS)
end)

gfx.setBackgroundColor(.2, .2, .2)
gfx.setBlendMode('alpha', 'premultiplied')

function love.draw()
	gfx.setColor(0, 0, 0)
	gfx.circle('fill', OBSX, OBSY, OBSR)

	for _, ball in ipairs(balls) do
		local body, color = ball[1], ball[2]
		local x, y = body:getPosition()
		gfx.setColor(color)
		gfx.draw(sprite, x, y, 0, 1, 1, BALLRADIUS, BALLRADIUS)
	end
end
